@@ -5,9 +5,11 @@ import datetime
 import json
 import logging
 import logging.handlers
+import os.path
+import shutil
 import sys
 import time
-from typing import Any, Dict, List
+from typing import Any, List
 
 import digitalocean
 import requests
@@ -17,9 +19,7 @@ from .__init__ import __basefilepath__, __version__
 logging.basicConfig(
     format="%(asctime)s [%(levelname)-5.5s]  %(message)s",
     handlers=[
-        logging.handlers.TimedRotatingFileHandler(
-            __basefilepath__ + "dobackup.log", when="W0", interval=2
-        ),
+        logging.handlers.TimedRotatingFileHandler(__basefilepath__ + "dobackup.log", when="W0", interval=2),
         logging.StreamHandler(sys.stdout),
     ],
     level="INFO",
@@ -28,57 +28,64 @@ log = logging.getLogger()
 
 
 def parse_args(argv: List[str]) -> argparse.Namespace:
-    parser = argparse.ArgumentParser(
-        description="Automated offline snapshots of digitalocean droplets"
-    )
+    parser = argparse.ArgumentParser(description="Automated Offline Or Live Snapshots Of Digitalocean Droplets")
     parser.add_argument(
         "token_id",
         nargs="?",
         help="Specify token number to be used, default=0, supply only if \
-    you have multiple DO accounts",
+    you have multiple Digitalocean accounts",
         default=0,
     )
-    parser.add_argument(
-        "-v", "-V", "--version", action="version", version="dobackup " + __version__
-    )
-    parser.add_argument(
-        "--init", dest="init", help="Save token to .token file", action="store_true"
-    )
-    parser.add_argument(
-        "-l", "--list-droplets", dest="list_droplets", help="List all droplets", action="store_true"
-    )
-    parser.add_argument(
+    parser.add_argument("-v", "-V", "--version", action="version", version="dobackup " + __version__)
+    parser.add_argument("--init", dest="init", help="Save token to .token file", action="store_true")
+
+    info_args = parser.add_argument_group("Informational Args", "Arguments That Display Information")
+    info_args.add_argument("-l", "--list-droplets", dest="list_droplets", help="List all droplets", action="store_true")
+    info_args.add_argument(
         "--list-backups",
         dest="list_backups",
         help='List all snapshots with "dobackup" in their name',
         action="store_true",
     )
-    parser.add_argument(
-        "-s", "--list-snaps", dest="list_snaps", help="List all snapshots", action="store_true"
+    info_args.add_argument("-s", "--list-snaps", dest="list_snaps", help="List all snapshots", action="store_true")
+    info_args.add_argument(
+        "--list-tagged", dest="list_tagged", help='List droplets using "--tag-name"', action="store_true"
     )
-    parser.add_argument(
-        "--list-tagged",
-        dest="list_tagged",
-        help='List droplets using "--tag-name"',
+    info_args.add_argument("--list-tags", dest="list_tags", help="List all used tags", action="store_true")
+    info_args.add_argument("--list-older-than", dest="list_older_than", type=int, help="List snaps older than, in days")
+
+    backup_args = parser.add_argument_group("Backup Args", "Arguments That Backup Or Restore Droplets")
+    backup_args.add_argument(
+        "--backup",
+        dest="backup",
+        type=str,
+        help="Shutdown, Backup (snapshot), Then Restart the droplet with given name or id",
+    )
+    backup_args.add_argument(
+        "--backup-all",
+        dest="backup_all",
+        help='Shutdown, Backup (snapshot), Then Restart all droplets with the given "--tag-name"',
         action="store_true",
     )
-    parser.add_argument(
-        "--list-tags", dest="list_tags", help="List all used tags", action="store_true"
-    )
-    parser.add_argument(
-        "--list-older-than", dest="list_older_than", type=int, help="List snaps older than, in days"
-    )
-    parser.add_argument(
-        "--tag-droplet",
-        dest="tag_droplet",
+    backup_args.add_argument(
+        "--live-backup",
+        dest="live_backup",
         type=str,
-        help="Add tag to the provided droplet name or id",
+        help="Backup (snapshot), the droplet with given name or id, without shutting it down",
     )
-    parser.add_argument(
-        "--untag-droplet",
-        dest="untag_droplet",
-        type=str,
-        help="Remove tag from the provided droplet name or id",
+    backup_args.add_argument(
+        "--live-backup-all",
+        dest="live_backup_all",
+        help='Backup (snapshot), all droplets with the given "--tag-name", without shutting them down',
+        action="store_true",
+    )
+
+    action_args = parser.add_argument_group("Action Args", "Arguments That Perform Actions")
+    action_args.add_argument(
+        "--tag-droplet", dest="tag_droplet", type=str, help="Add tag to the provided droplet name or id"
+    )
+    action_args.add_argument(
+        "--untag-droplet", dest="untag_droplet", type=str, help="Remove tag from the provided droplet name or id"
     )
     parser.add_argument(
         "--tag-name",
@@ -88,47 +95,24 @@ def parse_args(argv: List[str]) -> argparse.Namespace:
     default value is "dobackup"',
         default="dobackup",
     )
-    parser.add_argument(
-        "--delete-older-than",
-        dest="delete_older_than",
-        type=int,
-        help="Delete backups older than, in days",
+    action_args.add_argument(
+        "--delete-older-than", dest="delete_older_than", type=int, help="Delete backups older than, in days"
     )
-    parser.add_argument(
+    action_args.add_argument(
         "--delete-snap",
         dest="delete_snap",
         type=str,
-        help="Delete the snapshot with given name or id",
+        help="Snapshot(s) by name or id. e.g --delete-snap 111111 or --delete-snap '111111,222222,333333' "
     )
-    parser.add_argument(
-        "--backup",
-        dest="backup",
-        type=str,
-        help="Shutdown, Backup, Then Restart the droplet with given name or id",
-    )
-    parser.add_argument(
-        "--backup-all",
-        dest="backup_all",
-        help='Shutdown, Backup, Then Restart all droplets with "--tag-name"',
-        action="store_true",
-    )
-    parser.add_argument(
+    action_args.add_argument(
         "--shutdown", dest="shutdown", type=str, help="Shutdown, the droplet with given name or id"
     )
-    parser.add_argument(
-        "--powerup", dest="powerup", type=str, help="Powerup, the droplet with given name or id"
+    action_args.add_argument("--powerup", dest="powerup", type=str, help="Powerup, the droplet with given name or id")
+    backup_args.add_argument(
+        "--restore-droplet", dest="restore_drop", type=str, help="Restore, the droplet with given name or id"
     )
-    parser.add_argument(
-        "--restore-droplet",
-        dest="restore_drop",
-        type=str,
-        help="Restore, the droplet with given name or id",
-    )
-    parser.add_argument(
-        "--restore-to",
-        dest="restore_to",
-        type=str,
-        help="Snapshot id or name, to restore the droplet to",
+    backup_args.add_argument(
+        "--restore-to", dest="restore_to", type=str, help="Snapshot id or name, to restore the droplet to"
     )
     parser.add_argument(
         "--keep",
@@ -141,6 +125,221 @@ def parse_args(argv: List[str]) -> argparse.Namespace:
     return parser.parse_args(argv[1:])
 
 
+def run(
+    token_id: int,
+    init: bool,
+    list_droplets: bool,
+    list_backups: bool,
+    list_snaps: bool,
+    list_tagged: bool,
+    list_tags: bool,
+    list_older_than: int,
+    tag_droplet: str,
+    untag_droplet: str,
+    tag_name: str,
+    delete_older_than: int,
+    delete_snap: str,
+    backup: str,
+    backup_all: bool,
+    live_backup: str,
+    live_backup_all: bool,
+    shutdown: str,
+    powerup: str,
+    restore_drop: str,
+    restore_to: str,
+    keep: bool,
+) -> int:
+    try:
+        log.info("-------------------------START-------------------------\n")
+        if init:
+            if set_tokens() is False:
+                return 1
+            install_zsh_completion()
+
+        do_token = get_token(token_id)
+        if do_token == "":
+            return 1
+        manager = set_manager(do_token)
+
+        if list_droplets:
+            list_all_droplets(manager)
+        if list_backups:
+            list_taken_backups(manager, tag_name)
+        if list_snaps:
+            list_snapshots(manager)
+        if list_tagged:
+            tagged_droplets = get_tagged(manager, tag_name=tag_name)
+            log.info("Listing All The Tagged Droplets, With The Tag : '{}'".format(tag_name))
+            log.info(tagged_droplets)
+        if list_tags:
+            list_all_tags(manager)
+        if tag_droplet:
+            droplet = find_droplet(tag_droplet, manager)
+            if droplet is None:
+                return 1
+            do_tag_droplet(do_token, str(droplet.id), tag_name)
+            tagged_droplets = get_tagged(manager, tag_name=tag_name)
+            log.info("Now, Droplets Tagged With : '{}' Are :".format(tag_name))
+            log.info(tagged_droplets)
+        if untag_droplet:
+            droplet = find_droplet(untag_droplet, manager)
+            if droplet is None:
+                return 1
+            if do_untag_droplet(do_token, str(droplet.id), tag_name) is False:
+                return 1
+            tagged_droplets = get_tagged(manager, tag_name=tag_name)
+            log.info("Now, Droplets Tagged With : '{}' Are :".format(tag_name))
+            log.info(tagged_droplets)
+        if delete_older_than or delete_older_than == 0:  # even accept value 0
+            old_backups = find_old_backups(manager, delete_older_than, tag_name)
+            log.info(
+                "Snapshots Older Than {} Days, With '--{}--' In Their Name Are :"
+                " \n".format(delete_older_than, tag_name)
+            )
+            [log.info(str(x)) for x in old_backups]
+            if old_backups:  # not an empty list
+                [delete_snapshot(snap_x) for snap_x in old_backups]
+            else:
+                log.info("No Snapshot Is Old Enough To be Deleted")
+        if delete_snap:
+            # if multiple snaps are supplied
+            if "," in delete_snap:
+                multiple_snaps = delete_snap.split(",")
+                for each_snap in multiple_snaps:
+                    snap = find_snapshot(each_snap.strip(), manager, do_token)
+                    if snap:
+                        delete_snapshot(snap)
+            else:
+                snap = find_snapshot(delete_snap, manager, do_token)
+                if snap:
+                    delete_snapshot(snap)
+        if list_older_than or list_older_than == 0:
+            old_backups = find_old_backups(manager, list_older_than, tag_name)
+            log.info(
+                "Snapshots Older Than {!s} Days, With '--{}--' "
+                "In Their Name Are : \n".format(list_older_than, tag_name)
+            )
+            [log.info(str(x)) for x in old_backups]
+        if backup:
+            droplet = find_droplet(backup, manager)
+            if droplet is None:
+                return 1
+            original_status = droplet.status  # active or off
+            turn_it_off(droplet)
+            snap_action = start_backup(droplet, keep, tag_name)
+            snap_done = snap_completed(snap_action)
+            if original_status != "off":
+                turn_it_on(droplet)
+            if not snap_done:
+                log.error("SNAPSHOT FAILED {!s} {!s}".format(snap_action, droplet))
+        if backup_all:
+            # stores all {"snap_action": snap_action, "droplet_id": droplet}
+            snap_and_drop_ids = []
+            tagged_droplets = get_tagged(manager, tag_name=tag_name)
+
+            if tagged_droplets:  # doplets found with the --tag-name
+                for drop in tagged_droplets:
+                    droplet = send_command(5, manager, "get_droplet", drop.id)
+                    original_status = droplet.status  # active or off
+                    turn_it_off(droplet)
+                    snap_action = start_backup(droplet, keep, tag_name)
+                    snap_and_drop_ids.append(
+                        {"snap_action": snap_action, "droplet_id": droplet.id, "original_status": original_status}
+                    )
+                log.info("Backups Started, snap_and_drop_ids: {!s}".format(snap_and_drop_ids))
+                for snap_id_pair in snap_and_drop_ids:
+                    snap_done = snap_completed(snap_id_pair["snap_action"])
+                    # print("snap_action and droplet_id", snap_id_pair)
+                    if snap_id_pair["original_status"] != "off":
+                        turn_it_on(send_command(5, manager, "get_droplet", (snap_id_pair["droplet_id"])))
+                    if not snap_done:
+                        log.error("SNAPSHOT FAILED {!s} {!s}".format(snap_action, droplet))
+            else:  # no doplets with the --tag-name
+                log.warning("NO DROPLET FOUND WITH THE TAG NAME " + tag_name)
+        if live_backup:
+            droplet = find_droplet(live_backup, manager)
+            if droplet is None:
+                return 1
+            snap_action = start_backup(droplet, keep, tag_name)
+            snap_done = snap_completed(snap_action)
+            if not snap_done:
+                log.error("SNAPSHOT FAILED {!s} {!s}".format(snap_action, droplet))
+        if live_backup_all:
+            # stores all {"snap_action": snap_action, "droplet_id": droplet}
+            snap_and_drop_ids = []
+            tagged_droplets = get_tagged(manager, tag_name=tag_name)
+
+            if tagged_droplets:  # doplets found with the --tag-name
+                for drop in tagged_droplets:
+                    droplet = send_command(5, manager, "get_droplet", drop.id)
+                    original_status = droplet.status  # active or off
+                    snap_action = start_backup(droplet, keep, tag_name)
+                    snap_and_drop_ids.append(
+                        {"snap_action": snap_action, "droplet_id": droplet.id, "original_status": original_status}
+                    )
+                log.info("Backups Started, snap_and_drop_ids: {!s}".format(snap_and_drop_ids))
+                for snap_id_pair in snap_and_drop_ids:
+                    snap_done = snap_completed(snap_id_pair["snap_action"])
+                    # print("snap_action and droplet_id", snap_id_pair)
+                    if not snap_done:
+                        log.error("SNAPSHOT FAILED {!s} {!s}".format(snap_action, droplet))
+            else:  # no doplets with the --tag-name
+                log.warning("NO DROPLET FOUND WITH THE TAG NAME " + tag_name)
+        if shutdown:
+            droplet = find_droplet(shutdown, manager)
+            if droplet is None:
+                return 1
+            turn_it_off(droplet)
+        if powerup:
+            droplet = find_droplet(powerup, manager)
+            if droplet is None:
+                return 1
+            turn_it_on(droplet)
+        if restore_drop:
+            if restore_to:
+                droplet = find_droplet(restore_drop, manager)
+                if droplet is None:
+                    return 1
+                restore_droplet(droplet, restore_to, manager, do_token)
+            else:
+                log.warning("Please Use '--restore-to' To Provide The id Of " "Snapshot To Restore This Droplet To")
+
+        log.info("---------------------------END----------------------------\n\n")
+        return 0  # if all good, return 0
+    except Exception as e:
+        log.critical(e, exc_info=True)  # if errored at any time, mark CRITICAL and log traceback
+        return 1
+
+
+def main() -> int:
+    args = parse_args(sys.argv)
+    return_code = run(
+        args.token_id,
+        args.init,
+        args.list_droplets,
+        args.list_backups,
+        args.list_snaps,
+        args.list_tagged,
+        args.list_tags,
+        args.list_older_than,
+        args.tag_droplet,
+        args.untag_droplet,
+        args.tag_name,
+        args.delete_older_than,
+        args.delete_snap,
+        args.backup,
+        args.backup_all,
+        args.live_backup,
+        args.live_backup_all,
+        args.shutdown,
+        args.powerup,
+        args.restore_drop,
+        args.restore_to,
+        args.keep,
+    )
+    return return_code
+
+
 def set_tokens() -> bool:
     tokens = []
     token_dic = {}
@@ -171,21 +370,14 @@ def set_tokens() -> bool:
 
 
 def install_zsh_completion() -> None:
-    import os.path
-    import shutil
-
-    if os.path.exists(
-        os.path.join(os.path.expanduser("~"), ".oh-my-zsh/custom/plugins/zsh-completions/src/")
-    ):
+    if os.path.exists(os.path.join(os.path.expanduser("~"), ".oh-my-zsh/custom/plugins/zsh-completions/src/")):
         log.info("Zsh-completions path exists, installing completions file '_dobackup'")
         shutil.copy(
             __basefilepath__ + "_dobackup",
             os.path.join(os.path.expanduser("~"), ".oh-my-zsh/custom/plugins/zsh-completions/src/"),
         )
     else:
-        log.info(
-            "Zsh-completions with oh-my-zsh is not installed, can't use auto completions, but that's ok"
-        )
+        log.info("Zsh-completions with oh-my-zsh is not installed, can't use auto completions, but that's ok")
 
 
 def wait_for_action(an_action: digitalocean.Action, check_freq: int) -> bool:
@@ -229,8 +421,7 @@ def wait_for_action(an_action: digitalocean.Action, check_freq: int) -> bool:
         return False
 
 
-def send_command(
-        retries: int, obj: Any, method: str, *args, **kwargs) -> Any:
+def send_command(retries: int, obj: Any, method: str, *args, **kwargs) -> Any:
 
     # create dynamic function to run 'method' str as method
     # func = send_command(droplet, 'shutdown'), then func() == droplet.shutdown()
@@ -242,23 +433,19 @@ def send_command(
             # pass the args and kwargs through and run it
             command_output = run_command(*args, **kwargs)
         except json.decoder.JSONDecodeError:
-            log.warning(
-                "json.decoder.JSONDecodeError WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
+            log.warning("json.decoder.JSONDecodeError WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
             time.sleep(5)
             continue
         except digitalocean.baseapi.JSONReadError:
-            log.warning(
-                "json.decoder.JSONReadError WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
+            log.warning("json.decoder.JSONReadError WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
             time.sleep(5)
             continue
         except digitalocean.baseapi.DataReadError:
-            log.warning(
-                "json.decoder.DataReadError WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
+            log.warning("json.decoder.DataReadError WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
             time.sleep(5)
             continue
         except digitalocean.baseapi.Error:
-            log.warning(
-                "digitalocean.baseapi.Error, WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
+            log.warning("digitalocean.baseapi.Error, WHILE SENDING {!s}.{}(), TRYING AGAIN".format(obj, method))
             time.sleep(5)
             continue
         except ValueError:
@@ -276,29 +463,34 @@ def send_command(
 
 
 def turn_it_off(droplet: digitalocean.Droplet) -> bool:
-    log.info("Shutting Down : {!s}".format(droplet))
-    # send shutdown and capture that action's id
-    shut_action_id = send_command(5, droplet, "shutdown")["action"]["id"]
-    # print("shut_command: ", shut_command)
-    shut_action = send_command(5, droplet, "get_action", shut_action_id)
+    if droplet.status == "off":
+        log.info("The Droplet '{!s}' Is Already Powered Off".format(droplet))
+        return True
+    elif droplet.status == "active":
+        log.info("Shutting Down : {!s}".format(droplet))
+        # send shutdown and capture that action's id
+        shut_action_id = send_command(5, droplet, "shutdown")["action"]["id"]
+        # print("shut_command: ", shut_command)
+        shut_action = send_command(5, droplet, "get_action", shut_action_id)
 
-    log.debug("shut_action {!s} {!s}".format(shut_action, type(shut_action)))
-    shut_outcome = wait_for_action(shut_action, 3)
-    log.debug("shut_outcome {}".format(shut_outcome))
-    if shut_outcome:
-        for i in range(50):
-            time.sleep(3)
-            send_command(5, droplet, "load")  # refresh droplet data, retry 5 times
-            log.debug("droplet.status {} i== {!s}".format(droplet.status, i))
-            if droplet.status == "off":
-                log.info("Shutdown Completed " + str(droplet))
-                return True
-        log.error(
-            "SHUTDOWN FAILED, REPORTED 'shut_outcome'=='True' " + str(droplet) + str(shut_action)
-        )
-        return False
+        log.debug("shut_action {!s} {!s}".format(shut_action, type(shut_action)))
+        shut_outcome = wait_for_action(shut_action, 3)
+        log.debug("shut_outcome {}".format(shut_outcome))
+        if shut_outcome:
+            for i in range(50):
+                time.sleep(3)
+                send_command(5, droplet, "load")  # refresh droplet data, retry 5 times
+                log.debug("droplet.status {} i== {!s}".format(droplet.status, i))
+                if droplet.status == "off":
+                    log.info("Shutdown Completed " + str(droplet))
+                    return True
+            log.error("SHUTDOWN FAILED, REPORTED 'shut_outcome'=='True' " + str(droplet) + str(shut_action))
+            return False
+        else:
+            log.error("SHUTDOWN FAILED " + str(droplet) + str(shut_action))
+            return False
     else:
-        log.error("SHUTDOWN FAILED " + str(droplet) + str(shut_action))
+        log.error("'droplet.status' SHOULD BE EITHER 'off' OR 'active'")
         return False
 
 
@@ -306,16 +498,9 @@ def start_backup(droplet: digitalocean.Droplet, keep: bool, tag_name: str) -> di
     backup_str = "--" + tag_name + "--"
     if keep:
         backup_str = "--" + tag_name + "-keep--"
-    snap_name = (
-        droplet.name + backup_str + str(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
-    )
+    snap_name = droplet.name + backup_str + str(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
     # snap_name = droplet.name + "--dobackup--2018-05-02 12:37:52"
-    if droplet.status == "active":
-        turn_it_off(droplet)
-    elif droplet.status == "off":
-        log.info("The Droplet Is Already Off : " + str(droplet) + " Taking Snapshot")
-    else:
-        log.error("'droplet.status' SHOULD BE EITHER 'off' OR 'active'")
+
     log.info("Taking snapshot of " + droplet.name)
     # power_off is hard power off dont want that
     snap_action_id = send_command(5, droplet, "take_snapshot", snap_name, power_off=False)["action"]["id"]
@@ -335,25 +520,32 @@ def snap_completed(snap_action: digitalocean.Action) -> bool:
 
 
 def turn_it_on(droplet: digitalocean.Droplet) -> bool:
-    log.info("Powering Up {!s}".format(droplet))
-    power_up_action_id = send_command(5, droplet, "power_on")["action"]["id"]
-    power_up_action = send_command(5, droplet, "get_action", power_up_action_id)
-    log.debug("power_up_action " + str(power_up_action) + str(type(power_up_action)))
-    power_up_outcome = wait_for_action(power_up_action, 3)
-    log.debug("power_up_outcome " + str(power_up_outcome))
-    if power_up_outcome:
-        for i in range(5):
-            time.sleep(2)
-            droplet.load()  # refresh droplet data
-            log.debug("droplet.status " + droplet.status)
-            if droplet.status == "active":
-                log.info("Powered Back Up {!s}".format(droplet))
-                return True
-        log.critical("DID NOT POWER UP BUT REPORTED 'powered_up'=='True' " + str(droplet))
-        return False
+    if droplet.status == "active":
+        log.info("The Droplet '{!s}' Is Already Powered Up".format(droplet))
+        return True
+    elif droplet.status == "off":
+        log.info("Powering Up {!s}".format(droplet))
+        power_up_action_id = send_command(5, droplet, "power_on")["action"]["id"]
+        power_up_action = send_command(5, droplet, "get_action", power_up_action_id)
+        log.debug("power_up_action " + str(power_up_action) + str(type(power_up_action)))
+        power_up_outcome = wait_for_action(power_up_action, 3)
+        log.debug("power_up_outcome " + str(power_up_outcome))
+        if power_up_outcome:
+            for i in range(5):
+                time.sleep(2)
+                droplet.load()  # refresh droplet data
+                log.debug("droplet.status " + droplet.status)
+                if droplet.status == "active":
+                    log.info("Powered Back Up {!s}".format(droplet))
+                    return True
+            log.critical("DID NOT POWER UP BUT REPORTED 'powered_up'=='True' " + str(droplet))
+            return False
 
-    log.critical("DID NOT POWER UP " + str(droplet))
-    return False
+        log.critical("DID NOT POWER UP " + str(droplet))
+        return False
+    else:
+        log.error("'droplet.status' SHOULD BE EITHER 'off' OR 'active'")
+        return False
 
 
 def find_old_backups(manager: digitalocean.Manager, older_than: int, tag_name: str) -> List[digitalocean.Snapshot]:
@@ -408,12 +600,7 @@ def list_all_droplets(manager: digitalocean.Manager) -> None:
     log.info("Listing All Droplets:  ")
     log.info("<droplet-id>   <droplet-name>   <droplet-status>      <ip-addr>       <memory>\n")
     for droplet in my_droplets:
-        log.info(
-            str(droplet).ljust(40) +
-            droplet.status.ljust(12) +
-            droplet.ip_address.ljust(22) +
-            str(droplet.memory)
-        )
+        log.info(str(droplet).ljust(40) + droplet.status.ljust(12) + droplet.ip_address.ljust(22) + str(droplet.memory))
 
 
 def get_tagged(manager: digitalocean.Manager, tag_name: str) -> None:
@@ -473,32 +660,15 @@ def find_droplet(droplet_str: str, manager: digitalocean.Manager) -> digitalocea
 
 
 # Note: Snapshot.resource_id and Snapshot.id are str not int
-def find_snapshot(
-    snap_id_or_name: str, manager: digitalocean.Manager, do_token: str, droplet_id=000000
-) -> digitalocean.Snapshot:
-    snap_id_or_name = str(snap_id_or_name)  # for comparisions
+def find_snapshot(snap_id_or_name: str, manager: digitalocean.Manager, do_token: str) -> digitalocean.Snapshot:
+    snap_id_or_name = str(snap_id_or_name)  # for comparisons
     for snap in send_command(5, manager, "get_all_snapshots"):
-        # print(type(snap.resource_id), type(droplet_id))
-        if droplet_id == 000000:  # meaning, doesn't matter
-            if snap_id_or_name == str(snap.id) or snap_id_or_name == snap.name:
-                # snap_obj = digitalocean.Snapshot.get_object(do_token, snap.id)
-                snap_obj = send_command(5, digitalocean.Snapshot, "get_object", do_token, snap.id)
-                log.info("snap id and name {!s} {!s}".format(snap.id, snap.name))
-                return snap_obj
-        # to filter snapshots for a specific droplet
-        elif droplet_id == int(snap.resource_id):
-            log.info("snap id and name {!s} {!s}".format(snap.id, snap.name))
-            if snap_id_or_name == str(snap.id) or snap_id_or_name == snap.name:
-                snap_obj = send_command(5, digitalocean.Snapshot, "get_object", do_token, snap.id)
-                return snap_obj
-    if droplet_id == 000000:
-        log.error("NO SNAPSHOT FOUND WITH NAME OR ID OF {!s}, EXITING".format(snap_id_or_name))
-    else:
-        log.error(
-            "NO SNAPSHOT FOUND WITH NAME OR ID OF {!s} FOR DROPLET ID {!s}, EXITING".format(
-                snap_id_or_name, droplet_id
-            )
-        )
+        if snap_id_or_name == str(snap.id) or snap_id_or_name == snap.name:
+            # snap_obj = digitalocean.Snapshot.get_object(do_token, snap.id)
+            snap_obj = send_command(5, digitalocean.Snapshot, "get_object", do_token, snap.id)
+            # log.info("snap id and name {!s} {!s}".format(snap.id, snap.name))
+            return snap_obj
+    log.error("NO SNAPSHOT FOUND WITH NAME OR ID OF {!s}, EXITING".format(snap_id_or_name))
 
 
 def list_taken_backups(manager: digitalocean.Manager, tag_name: str) -> None:
@@ -506,7 +676,10 @@ def list_taken_backups(manager: digitalocean.Manager, tag_name: str) -> None:
     tag_str_keep = "--" + tag_name + "-keep--"
     log.info(
         "The Backups Taken With dobackup using tag '{}' Are : <snapshot-name>\
-        <snapshot-id>\n".format(tag_name))
+        <snapshot-id>\n".format(
+            tag_name
+        )
+    )
     backups = []
     for snap in send_command(5, manager, "get_all_snapshots"):
         if tag_str in snap.name or tag_str_keep in snap.name:
@@ -517,16 +690,13 @@ def list_taken_backups(manager: digitalocean.Manager, tag_name: str) -> None:
 
 
 def restore_droplet(
-    droplet: digitalocean.Droplet,
-    snapshot: digitalocean.Snapshot,
-    manager: digitalocean.Manager,
-    do_token: str,
+    droplet: digitalocean.Droplet, snapshot: digitalocean.Snapshot, manager: digitalocean.Manager, do_token: str
 ):
-    snap = find_snapshot(snapshot, manager, do_token, droplet_id=droplet.id)
+    snap = find_snapshot(snapshot, manager, do_token)
 
     if snap:
-        log.info(str(snap) + " Is A Valid Snapshot For " + droplet.name + "\n")
-        confirmation = input("Are You Sure You Want To Restore ? (if so, type 'yes') ")
+        log.info(str(snap) + " Is A Valid Snapshot\n")
+        confirmation = input(f"Are You Sure You Want To Restore {droplet.name}? (if so, type 'yes') ")
         if confirmation.lower() == "yes":
             log.info("Starting Restore Process")
             restore_act_id = send_command(5, droplet, "restore", (int(snap.id)))["action"]["id"]
@@ -538,182 +708,7 @@ def restore_droplet(
                 log.error("RESTORE FAILED " + str(restore_act))
 
     if not snap:
-        log.error(str(snapshot) + " IS NOT A VALID SNAPSHOT FOR " + droplet.name)
-
-
-def run(
-    token_id: int,
-    init: bool,
-    list_droplets: bool,
-    list_backups: bool,
-    list_snaps: bool,
-    list_tagged: bool,
-    list_tags: bool,
-    list_older_than: int,
-    tag_droplet: str,
-    untag_droplet: str,
-    tag_name: str,
-    delete_older_than: int,
-    delete_snap: str,
-    backup: str,
-    backup_all: bool,
-    shutdown: str,
-    powerup: str,
-    restore_drop: str,
-    restore_to: str,
-    keep: bool,
-) -> int:
-    try:
-        log.info("-------------------------START-------------------------\n\n")
-        if init:
-            if set_tokens() is False:
-                return 1
-            install_zsh_completion()
-
-        do_token = get_token(token_id)
-        if do_token == "":
-            return 1
-        manager = set_manager(do_token)
-
-        if list_droplets:
-            list_all_droplets(manager)
-        if list_backups:
-            list_taken_backups(manager, tag_name)
-        if list_snaps:
-            list_snapshots(manager)
-        if list_tagged:
-            tagged_droplets = get_tagged(manager, tag_name=tag_name)
-            log.info("Listing All The Tagged Droplets, with the tag of : " + tag_name)
-            log.info(tagged_droplets)
-        if list_tags:
-            list_all_tags(manager)
-        if tag_droplet:
-            droplet = find_droplet(tag_droplet, manager)
-            if droplet is None:
-                return 1
-            do_tag_droplet(do_token, str(droplet.id), tag_name)
-            tagged_droplets = get_tagged(manager, tag_name=tag_name)
-            log.info("Now, Droplets Tagged With : {} Are :".format(tag_name))
-            log.info(tagged_droplets)
-        if untag_droplet:
-            droplet = find_droplet(untag_droplet, manager)
-            if droplet is None:
-                return 1
-            if do_untag_droplet(do_token, str(droplet.id), tag_name) is False:
-                return 1
-            tagged_droplets = get_tagged(manager, tag_name=tag_name)
-            log.info("Now, droplets tagged with : " + tag_name + " are :")
-            log.info(tagged_droplets)
-        if delete_older_than or delete_older_than == 0:  # even accept value 0
-            old_backups = find_old_backups(manager, delete_older_than, tag_name)
-            log.info(
-                "Snapshots Older Than {} Days, With '--{}--' In Their Name Are :"
-                " \n".format(delete_older_than, tag_name)
-            )
-            [log.info(str(x)) for x in old_backups]
-            if old_backups:  # not an empty list
-                [delete_snapshot(snap_x) for snap_x in old_backups]
-            else:
-                log.info("No Snapshot Is Old Enough To be Deleted")
-        if delete_snap:
-            snap = find_snapshot(delete_snap, manager, do_token)
-            if snap:
-                delete_snapshot(snap)
-        if list_older_than or list_older_than == 0:
-            old_backups = find_old_backups(manager, list_older_than, tag_name)
-            log.info(
-                "Snapshots Older Than {!s} Days, With '--{}--' "
-                "In Their Name Are : \n".format(list_older_than, tag_name)
-            )
-            [log.info(str(x)) for x in old_backups]
-        if backup:
-            droplet = find_droplet(backup, manager)
-            if droplet is None:
-                return 1
-            original_status = droplet.status  # active or off
-            snap_action = start_backup(droplet, keep, tag_name)
-            snap_done = snap_completed(snap_action)
-            if original_status != "off":
-                turn_it_on(droplet)
-            if not snap_done:
-                log.error("SNAPSHOT FAILED {!s} {!s}".format(snap_action, droplet))
-        if backup_all:
-            # stores all {"snap_action": snap_action, "droplet_id": droplet}
-            snap_and_drop_ids = []
-            tagged_droplets = get_tagged(manager, tag_name=tag_name)
-
-            if tagged_droplets:  # doplets found with the --tag-name
-                for drop in tagged_droplets:
-                    droplet = send_command(5, manager, "get_droplet", drop.id)
-                    original_status = droplet.status  # active or off
-                    snap_action = start_backup(droplet, keep, tag_name)
-                    snap_and_drop_ids.append(
-                        {"snap_action": snap_action, "droplet_id": droplet.id, "status": original_status})
-                log.info("Backups Started, snap_and_drop_ids: {!s}".format(snap_and_drop_ids))
-                for snap_id_pair in snap_and_drop_ids:
-                    snap_done = snap_completed(snap_id_pair["snap_action"])
-                    # print("snap_action and droplet_id", snap_id_pair)
-                    if snap_id_pair["status"] != "off":
-                        turn_it_on(send_command(5, manager, "get_droplet", (snap_id_pair["droplet_id"])))
-                    if not snap_done:
-                        log.error("SNAPSHOT FAILED {!s} {!s}".format(snap_action, droplet))
-            else:  # no doplets with the --tag-name
-                log.warning("NO DROPLET FOUND WITH THE TAG NAME " + tag_name)
-        if shutdown:
-            droplet = find_droplet(shutdown, manager)
-            if droplet is None:
-                return 1
-            turn_it_off(droplet)
-        if powerup:
-            droplet = find_droplet(powerup, manager)
-            if droplet is None:
-                return 1
-            turn_it_on(droplet)
-        if restore_drop:
-            if restore_to:
-                droplet = find_droplet(restore_drop, manager)
-                if droplet is None:
-                    return 1
-                restore_droplet(droplet, restore_to, manager, do_token)
-            else:
-                log.warning(
-                    "Please Use '--restore-to' To Provide The id Of "
-                    "Snapshot To Restore This Droplet To"
-                )
-
-        log.info("---------------------------END----------------------------")
-        log.info("\n\n")
-        return 0  # if all good, return 0
-    except Exception as e:
-        log.critical(e, exc_info=True)  # if errored at any time, mark CRITICAL and log traceback
-        return 1
-
-
-def main() -> int:
-    args = parse_args(sys.argv)
-    return_code = run(
-        args.token_id,
-        args.init,
-        args.list_droplets,
-        args.list_backups,
-        args.list_snaps,
-        args.list_tagged,
-        args.list_tags,
-        args.list_older_than,
-        args.tag_droplet,
-        args.untag_droplet,
-        args.tag_name,
-        args.delete_older_than,
-        args.delete_snap,
-        args.backup,
-        args.backup_all,
-        args.shutdown,
-        args.powerup,
-        args.restore_drop,
-        args.restore_to,
-        args.keep,
-    )
-    return return_code
+        log.error(str(snapshot) + " IS NOT A VALID SNAPSHOT")
 
 
 if __name__ == "__main__":
